@@ -1465,10 +1465,10 @@ exit( 1 );
 
 void freeconstants()
 {
-	FreeDoubleMtx( n_disLN );
-	FreeIntMtx( n_dis );
-	FreeIntMtx( n_disFFT );
-	FreeDoubleMtx( n_dis_consweight_multi );
-	FreeIntMtx( amino_dis );
-	FreeDoubleMtx( amino_dis_consweight_multi );
+	if( n_disLN ) FreeDoubleMtx( n_disLN ); n_disLN = NULL;
+	if( n_dis ) FreeIntMtx( n_dis ); n_dis = NULL;
+	if( n_disFFT ) FreeIntMtx( n_disFFT ); n_disFFT = NULL;
+	if( n_dis_consweight_multi ) FreeDoubleMtx( n_dis_consweight_multi ); n_dis_consweight_multi = NULL;
+	if( amino_dis ) FreeIntMtx( amino_dis ); amino_dis = NULL;
+	if( amino_dis_consweight_multi ) FreeDoubleMtx( amino_dis_consweight_multi ); amino_dis_consweight_multi = NULL;
 }
