@@ -210,6 +210,7 @@ extern int countATGC( char *s, int *total );
 extern void getnumlen( FILE *fp );
 extern void getnumlen_casepreserve( FILE *fp, int *nlenmin );
 extern void getnumlen_nogap( FILE *fp, int *nlenmin );
+extern void getnumlen_nogap_countn( FILE *fp, int *nlenmin, double *nfreq );
 extern void WriteGapFill( FILE *fp, int locnjob, char name[][B], int nlen[M], char **aseq );
 extern void writeDataforgaln( FILE *fp, int locnjob, char **name, int *nlen, char **aseq );
 extern void writeData( FILE *fp, int locnjob, char name[][B], int nlen[], char **aseq );
@@ -394,8 +395,12 @@ extern double D__align_gapmap( char **seq1, char **seq2, double *eff1, double *e
 extern void stringshuffle( int *ary, int size );
 extern void topolorder( int n, int *order, int *posinorder, int ***topol, Treedep *dep, int pos, int child );
 extern int commonsextet_p( int *table, int *pointt );
-extern void compacttree_memsave( int nseq, double **partmtx, int *nearest, double *mindist, int **pointt, int *selfscore, char **seq, int **skiptable, int ***topol, double **len, char **name, int *nlen, Treedep *dep, int treeout, int howcompact );
+extern void compacttree_memsaveselectable( int nseq, double **partmtx, int *nearest, double *mindist, int **pointt, int *selfscore, char **seq, int **skiptable, int ***topol, double **len, char **name, int *nlen, Treedep *dep, int treeout, int howcompact, int memsave );
 extern double distcompact( int len1, int len2, int *table1, int *point2, int ss1, int ss2 );
 extern double distcompact_msa( char *seq1, char *seq2, int *skiptable1, int *skiptable2, int ss1, int ss2 );
 extern void fillimp( double **impmtx, double *imp, int clus1, int clus2, int lgth1, int lgth2, char **seq1, char **seq2, double *eff1, double *eff2, double *eff1_kozo, double *eff2_kozo, LocalHom ***localhom, char *swaplist, int forscore, int *orinum1, int *orinum2 );
 extern int pairlocalalign( int ngui, int lgui, char **namegui, char **seqgui, double **distancemtx, LocalHom **localhomtable, int argc, char **argv );
+extern char creverse( char f );
+#ifndef mingw
+extern void setstacksize( rlim_t );
+#endif
