@@ -427,6 +427,30 @@ void FreeIntCub( int ***cub )
 	free( cub );
 }
 
+double **AllocateDoubleHalfMtx( int ll1 )
+{
+	double **mtx;
+	int i;
+
+	mtx = (double **)calloc( (unsigned int)ll1+1, sizeof( double * ) );
+	if( mtx == NULL )
+	{
+		fprintf( stderr, "Allocation error ( %d double halfmtx )\n", ll1 );
+		exit( 1 );
+	}
+	for( i=0; i<ll1; i++ )
+	{
+		mtx[i] = (double *)calloc( ll1-i, sizeof( double ) );
+		if( !mtx[i] )
+		{
+			fprintf( stderr, "Allocation error( %d double halfmtx )\n", ll1 );
+			exit( 1 );
+		}
+	}
+	mtx[ll1] = NULL;
+	return( mtx );
+}
+
 double **AllocateDoubleMtx( int ll1, int ll2 )
 {
 	int i;
@@ -447,6 +471,16 @@ double **AllocateDoubleMtx( int ll1, int ll2 )
 	return mtx;
 }
 
+void FreeDoubleHalfMtx( double **mtx, int n )
+{
+	int i;
+
+	for( i=0; i<n; i++ ) 
+	{
+		if( mtx[i] ) FreeFloatVec( mtx[i] ); mtx[i] = NULL;
+	}
+	free( mtx );
+}
 void FreeDoubleMtx( double **mtx )
 {
 	int i;
