@@ -34,7 +34,9 @@ void FreeIntCub( int *** );
 double *AllocateDoubleVec( int );
 void FreeDoubleVec( double * );
 
+double **AllocateDoubleHalfMtx( int );
 double **AllocateDoubleMtx( int, int );
+void FreeDoubleHalfMtx( double **, int );
 void FreeDoubleMtx( double ** );
 
 double ***AllocateDoubleCub( int, int, int );
