@@ -3,7 +3,7 @@
 #define DEBUG 0
 #define IODEBUG 0
 #define SCOREOUT 0
-
+#define SHISHAGONYU 0 // for debug
 
 #define NODIST -9999
 
@@ -20,6 +20,7 @@ static int lastm;
 static int lastsubopt;
 static int lastonce;
 static int usenaivescoreinsteadofalignmentscore;
+static int specifictarget;
 
 typedef struct _lastres
 {
@@ -72,6 +73,7 @@ typedef struct _thread_arg
 	char ***bpp;
 	Lastresx **lastresx;
 	int alloclen;
+	int *targetmap;
 	pthread_mutex_t *mutex_counter;
 	pthread_mutex_t *mutex_stdout;
 } thread_arg_t;
@@ -128,7 +130,7 @@ static int removex( char *d, char *m )
 	return( val );
 }
 
-static void putlocalhom_last( char *s1, char *s2, LocalHom *localhompt, Lastresx *lastresx )
+static void putlocalhom_last( char *s1, char *s2, LocalHom *localhompt, Lastresx *lastresx, char korh )
 {
 	char *pt1, *pt2;
 	int naln, nreg;
@@ -172,6 +174,7 @@ static void putlocalhom_last( char *s1, char *s2, LocalHom *localhompt, Lastresx
 			tmppt->start2 = rpt2->start;
 			tmppt->end1   = rpt1->end;
 			tmppt->end2   = rpt2->end;
+			tmppt->korh   = 'h';
 			if( rpt1 == apt->reg1 ) localhompt0 = tmppt; // ?
 	
 //			fprintf( stderr, "in putlocalhom, reg1: %d-%d (nreg=%d)\n", rpt1->start, rpt1->end, lastresx->nreg );
@@ -1567,7 +1570,7 @@ static void preparebpp( int nseq, char ***bpp )
 	fclose( fp );
 }
 
-void arguments( int argc, char *argv[] )
+static void arguments( int argc, char *argv[] )
 {
     int c;
 
@@ -1615,7 +1618,7 @@ void arguments( int argc, char *argv[] )
 	stdout_dist = 0;
 	store_dist = 1;
 	store_localhom = 1;
-	dorp = NOTSPECIFIED;
+//	dorp = NOTSPECIFIED;
 	ppenalty = NOTSPECIFIED;
 	ppenalty_OP = NOTSPECIFIED;
 	ppenalty_ex = NOTSPECIFIED;
@@ -1631,16 +1634,22 @@ void arguments( int argc, char *argv[] )
 	RNApthr = NOTSPECIFIED;
 	specificityconsideration = 0.0;
 	usenaivescoreinsteadofalignmentscore = 0;
+	specifictarget = 0;
+	nwildcard = 0;
 
+//	reporterr( "argc=%d\n", argc );
+//	reporterr( "*argv=%s\n", *argv );
+//	reporterr( "(*argv)[0]=%c\n", (*argv)[0] );
     while( --argc > 0 && (*++argv)[0] == '-' )
 	{
+//		reporterr( "(*argv)[0] in while loop = %s\n", (*argv) );
         while ( ( c = *++argv[0] ) )
 		{
             switch( c )
             {
 				case 'i':
 					inputfile = *++argv;
-					fprintf( stderr, "inputfile = %s\n", inputfile );
+//					fprintf( stderr, "inputfile = %s\n", inputfile );
 					--argc;
 					goto nextoption;
 				case 'f':
@@ -1682,14 +1691,14 @@ void arguments( int argc, char *argv[] )
 					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = JTT;
-					fprintf( stderr, "jtt %d\n", pamN );
+//					fprintf( stderr, "jtt %d\n", pamN );
 					--argc;
 					goto nextoption;
 				case 'm':
 					pamN = myatoi( *++argv );
 					scoremtx = 0;
 					TMorJTT = TM;
-					fprintf( stderr, "TM %d\n", pamN );
+//					fprintf( stderr, "TM %d\n", pamN );
 					--argc;
 					goto nextoption;
 #if 0
@@ -1718,12 +1727,15 @@ void arguments( int argc, char *argv[] )
 					goto nextoption;
 				case 'C':
 					nthread = myatoi( *++argv );
-					fprintf( stderr, "nthread = %d\n", nthread );
+//					fprintf( stderr, "nthread = %d\n", nthread );
 					--argc; 
+#ifndef enablemultithread
+					nthread = 0;
+#endif
 					goto nextoption;
 				case 'I':
 					nadd = myatoi( *++argv );
-					fprintf( stderr, "nadd = %d\n", nadd );
+//					fprintf( stderr, "nadd = %d\n", nadd );
 					--argc;
 					goto nextoption;
 				case 'w':
@@ -1852,6 +1864,12 @@ void arguments( int argc, char *argv[] )
 				case 'y':
 					divpairscore = 1;
 					break;
+				case '=':
+					specifictarget = 1;
+					break;
+				case ':':
+					nwildcard = 1;
+					break;
 /* Modified 01/08/27, default: user tree */
 				case 'J':
 					tbutree = 0;
@@ -1893,7 +1911,7 @@ void arguments( int argc, char *argv[] )
     }
     if( argc != 0 ) 
     {
-        fprintf( stderr, "options: Check source file !\n" );
+        fprintf( stderr, "pairlocalalign options: Check source file !\n" );
         exit( 1 );
     }
 	if( tbitr == 1 && outgap == 0 )
@@ -1956,6 +1974,7 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 	char ***bpp = targ->bpp;
 	Lastresx **lastresx = targ->lastresx;
 	int alloclen = targ->alloclen;
+	int *targetmap = targ->targetmap;
 
 //	fprintf( stderr, "thread %d start!\n", thread_no );
 
@@ -1976,6 +1995,7 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 	if( alg == 'Y' || alg == 'r' ) ilim = njob - nadd;
 	else ilim = njob - 1;
 
+
 	while( 1 )
 	{
 		pthread_mutex_lock( targ->mutex_counter );
@@ -2038,7 +2058,7 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 			if( store_dist )
 			{
 				if( alg == 'Y' || alg == 'r' ) distancemtx[i][j-(njob-nadd)] = 3.0;
-				else distancemtx[i][j] = 3.0;
+				else distancemtx[i][j-i] = 3.0;
 			}
 			if( stdout_dist) 
 			{
@@ -2111,7 +2131,8 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 						}
 						else
 						{
-							if( store_localhom )
+//							if( store_localhom )
+							if( store_localhom && ( targetmap[i] != -1 || targetmap[j] != -1 ) )
 							{
 								pscore = L__align11( n_dis_consweight_multi, 0.0, mseq1, mseq2, alloclen, &off1, &off2 );
 								if( thereisx ) pscore = L__align11_noalign( n_dis_consweight_multi, distseq1, distseq2 ); // uwagaki
@@ -2165,7 +2186,8 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 					}
 					else
 					{
-						if( store_localhom )
+//						if( store_localhom )
+						if( store_localhom && ( targetmap[i] != -1 || targetmap[j] != -1 ) )
 						{
 							pscore = G__align11( n_dis_consweight_multi, mseq1, mseq2, alloclen, outgap, outgap );
 							if( thereisx ) pscore = G__align11_noalign( n_dis_consweight_multi, penalty, penalty_ex, distseq1, distseq2, alloclen ); // uwagaki
@@ -2271,16 +2293,43 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 			{
 				if( !store_localhom )
 					;
+				else if( specifictarget && targetmap[i] == -1 && targetmap[j] == -1)
+					;
 				else if( alg == 'R' )
-					putlocalhom_last( mseq1[0], mseq2[0], localhomtable[i]+j, lastresx[i]+j );
+					putlocalhom_last( mseq1[0], mseq2[0], localhomtable[i]+j, lastresx[i]+j, 'h' );
 				else if( alg == 'r' )
-					putlocalhom_last( mseq1[0], mseq2[0], localhomtable[i]+j-(njob-nadd), lastresx[i]+j-(njob-nadd) );// ?????
+					putlocalhom_last( mseq1[0], mseq2[0], localhomtable[i]+j-(njob-nadd), lastresx[i]+j-(njob-nadd), 'h' );// ?????
 				else if( alg == 'H' )
-					putlocalhom_ext( mseq1[0], mseq2[0], localhomtable[i]+j, off1, off2, (int)pscore, strlen( mseq1[0] ) );
+					putlocalhom_ext( mseq1[0], mseq2[0], localhomtable[i]+j, off1, off2, (int)pscore, strlen( mseq1[0] ), 'h' );
 				else if( alg == 'Y' )
-					putlocalhom2( mseq1[0], mseq2[0], localhomtable[i]+j-(njob-nadd), off1, off2, (int)pscore, strlen( mseq1[0] ) );
-				else if( alg != 'S' && alg != 'V' )
-					putlocalhom2( mseq1[0], mseq2[0], localhomtable[i]+j, off1, off2, (int)pscore, strlen( mseq1[0] ) );
+					putlocalhom2( mseq1[0], mseq2[0], localhomtable[i]+j-(njob-nadd), off1, off2, (int)pscore, strlen( mseq1[0] ), 'h' );
+				else if( !specifictarget && alg != 'S' && alg != 'V' )
+					putlocalhom2( mseq1[0], mseq2[0], localhomtable[i]+j-i, off1, off2, (int)pscore, strlen( mseq1[0] ), 'h' );
+				else
+//					putlocalhom2( mseq1[0], mseq2[0], localhomtable[i]+j, off1, off2, (int)pscore, strlen( mseq1[0] ) );
+				{
+					if( targetmap[i] != -1 && targetmap[j] != -1 )
+					{
+						putlocalhom2( mseq2[0], mseq1[0], localhomtable[targetmap[j]]+i, off2, off1, (int)pscore, strlen( mseq2[0] ), 'h' );
+						putlocalhom2( mseq1[0], mseq2[0], localhomtable[targetmap[i]]+j, off1, off2, (int)pscore, strlen( mseq1[0] ), 'h' ); // sukoshi muda.
+					}
+					else if( targetmap[j] != -1 )
+						putlocalhom2( mseq2[0], mseq1[0], localhomtable[targetmap[j]]+i, off2, off1, (int)pscore, strlen( mseq2[0] ), 'h' );
+					else if( targetmap[i] != -1 )
+						putlocalhom2( mseq1[0], mseq2[0], localhomtable[targetmap[i]]+j, off1, off2, (int)pscore, strlen( mseq1[0] ), 'h' );
+#if 0
+					if( targetmap[i] != -1 )
+						putlocalhom2( mseq1[0], mseq2[0], localhomtable[targetmap[i]]+j, off1, off2, (int)pscore, strlen( mseq1[0] ), 'h' );
+					
+					else if( targetmap[j] != -1 )
+						putlocalhom2( mseq2[0], mseq1[0], localhomtable[targetmap[j]]+i, off2, off1, (int)pscore, strlen( mseq2[0] ), 'h' );
+#endif
+					else
+					{
+						reporterr( "okashii\n" );
+						exit( 1 );
+					}
+				}
 			}
 			pscore = score2dist( pscore, selfscore[i], selfscore[j] );
 
@@ -2320,33 +2369,77 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 		if( store_dist )
 		{
 			if( alg == 'Y' || alg == 'r' ) distancemtx[i][j-(njob-nadd)] = pscore;
-			else distancemtx[i][j] = pscore;
+			else distancemtx[i][j-i] = pscore;
 		}
 	}
 }
 #endif
 
-static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **dseq, int *thereisxineachseq, char **mseq1, char **mseq2, int alloclen, Lastresx **lastresx )
+static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **dseq, int *thereisxineachseq, char **mseq1, char **mseq2, int alloclen, Lastresx **lastresx, double **distancemtx, LocalHom **localhomtable, int ngui )
 {
 	int i, j, ilim, jst, jj;
 	int off1, off2, dum1, dum2, thereisx;
 	double pscore = 0.0; // by D.Mathog
 	FILE *hat2p, *hat3p;
-	double **distancemtx;
+//	double **distancemtx;
 	double *selfscore;
 	double *effarr1;
 	double *effarr2;
 	char *pt;
 	char *hat2file = "hat2";
-	LocalHom **localhomtable = NULL, *tmpptr;
+//	LocalHom **localhomtable = NULL, 
+	LocalHom *tmpptr;
 	int intdum;
 	char ***bpp = NULL; // mxscarna no toki dake
 	char **distseq1, **distseq2;
 	char **dumseq1, **dumseq2;
 	double dist;
 	double scoreoffset;
+	int ntarget;
+	int *targetmap, *targetmapr;
 
-	if( store_localhom )
+
+	if( specifictarget )
+	{
+		targetmap = calloc( njob, sizeof( int ) );
+		ntarget = 0;
+		for( i=0; i<njob; i++ )
+		{
+			targetmap[i] = -1;
+			if( !strncmp( name[i]+1, "_focus_", 7 ) )
+				targetmap[i] = ntarget++;
+		}
+		targetmapr = calloc( ntarget, sizeof( int ) );
+		for( i=0; i<njob; i++ )
+			if( targetmap[i] != -1 ) targetmapr[targetmap[i]] = i;
+
+		if( ntarget == 0 )
+		{
+			reporterr( "\n\nAdd '>_focus_' to the title lines of the sequences to be focused on.\n\n" );
+			exit( 1 );
+		}
+		else
+		{
+			reporterr( "nfocus = %d \n", ntarget );
+		}
+	}
+	else
+	{
+		ntarget = njob;
+		targetmap = calloc( njob, sizeof( int ) );
+		targetmapr = calloc( njob, sizeof( int ) );
+		for( i=0; i<njob; i++ )
+			targetmap[i] = targetmapr[i] = i;
+	}
+
+#if 0
+	for( i=0; i<njob; i++ )
+		reporterr( "targetmap[%d] = %d\n", i, targetmap[i] );
+	for( i=0; i<ntarget; i++ )
+		reporterr( "targetmapr[%d] = %d\n", i, targetmapr[i] );
+#endif
+
+	if( store_localhom && localhomtable == NULL )
 	{
 		if( alg == 'Y' || alg == 'r' )
 		{
@@ -2355,7 +2448,7 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **d
 		}
 		else
 		{
-			ilim = njob;
+			ilim = ntarget;
 			jst = njob;
 		}
 		localhomtable = (LocalHom **)calloc( ilim, sizeof( LocalHom *) );
@@ -2371,16 +2464,24 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **d
 				localhomtable[i][j].opt = -1.0;
 				localhomtable[i][j].next = NULL;
 				localhomtable[i][j].nokori = 0;
+				localhomtable[i][j].extended = -1;
+				localhomtable[i][j].last = localhomtable[i]+j;
+				localhomtable[i][j].korh = 'h';
 			}
+			if( !specifictarget && alg != 'Y' && alg != 'r' ) jst--;
 		}
 	}
 
 	if( store_dist )
 	{
-		if( alg == 'Y' || alg == 'r' )
-			distancemtx = AllocateDoubleMtx( njob, nadd );
-		else
-			distancemtx = AllocateDoubleMtx( njob, njob );
+		if( ngui == 0 )
+		{
+			if( alg == 'Y' || alg == 'r' )
+				distancemtx = AllocateDoubleMtx( njob, nadd );
+			else
+				distancemtx = AllocateDoubleHalfMtx( njob );
+//				distancemtx = AllocateDoubleMtx( njob, njob );
+		}
 	}
 	else distancemtx = NULL;
 
@@ -2408,6 +2509,7 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **d
 
 //	writePre( njob, name, nlen, aseq, 0 );
 
+	reporterr( "All-to-all alignment.\n" );
 	if( alg == 'R' )
 	{
 		fprintf( stderr, "Calling last (http://last.cbrc.jp/)\n" );
@@ -2515,6 +2617,7 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **d
 			targ[i].bpp = bpp; 
 			targ[i].lastresx = lastresx;
 			targ[i].alloclen = alloclen;
+			targ[i].targetmap = targetmap;
 			targ[i].mutex_counter = &mutex_counter;
 			targ[i].mutex_stdout = &mutex_stdout;
 
@@ -2557,7 +2660,7 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **d
 					if( store_dist ) 
 					{
 						if( alg == 'Y' || alg == 'r' ) distancemtx[i][j-(njob-nadd)] = 3.0;
-						else distancemtx[i][j] = 3.0;
+						else distancemtx[i][j-i] = 3.0;
 					}
 					if( stdout_dist) fprintf( stdout, "%d %d d=%.3f\n", i+1, j+1, 3.0 );
 					continue;
@@ -2614,7 +2717,8 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **d
 							}
 							else
 							{
-								if( store_localhom )
+//								if( store_localhom )
+								if( store_localhom && ( targetmap[i] != -1 || targetmap[j] != -1 ) )
 								{
 									pscore = G__align11( n_dis_consweight_multi, mseq1, mseq2, alloclen, outgap, outgap );
 									if( thereisx ) pscore = G__align11_noalign( n_dis_consweight_multi, penalty, penalty_ex, distseq1, distseq2, alloclen ); // uwagaki
@@ -2695,7 +2799,8 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **d
 								}
 								else
 								{
-									if( store_localhom )
+//									if( store_localhom )
+									if( store_localhom && ( targetmap[i] != -1 || targetmap[j] != -1 ) )
 									{
 										pscore = L__align11( n_dis_consweight_multi, 0.0, mseq1, mseq2, alloclen, &off1, &off2 ); // all pair
 										if( thereisx ) pscore = L__align11_noalign( n_dis_consweight_multi, distseq1, distseq2 ); // all pair
@@ -2785,16 +2890,35 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **d
 					{
 						if( !store_localhom )
 							;
+						else if( specifictarget && targetmap[i] == -1 && targetmap[j] == -1)
+							;
 						else if( alg == 'R' )
-							putlocalhom_last( mseq1[0], mseq2[0], localhomtable[i]+j, lastresx[i]+j );
+							putlocalhom_last( mseq1[0], mseq2[0], localhomtable[i]+j, lastresx[i]+j, 'h' );
 						else if( alg == 'r' )
-							putlocalhom_last( mseq1[0], mseq2[0], localhomtable[i]+j-(njob-nadd), lastresx[i]+j-(njob-nadd) );// ?????
+							putlocalhom_last( mseq1[0], mseq2[0], localhomtable[i]+j-(njob-nadd), lastresx[i]+j-(njob-nadd), 'h' );// ?????
 						else if( alg == 'H' )
-							putlocalhom_ext( mseq1[0], mseq2[0], localhomtable[i]+j, off1, off2, (int)pscore, strlen( mseq1[0] ) );
+							putlocalhom_ext( mseq1[0], mseq2[0], localhomtable[i]+j, off1, off2, (int)pscore, strlen( mseq1[0] ), 'h' );
 						else if( alg == 'Y' )
-							putlocalhom2( mseq1[0], mseq2[0], localhomtable[i]+j-(njob-nadd), off1, off2, (int)pscore, strlen( mseq1[0] ) );
-						else if( alg != 'S' && alg != 'V' )
-							putlocalhom2( mseq1[0], mseq2[0], localhomtable[i]+j, off1, off2, (int)pscore, strlen( mseq1[0] ) );
+							putlocalhom2( mseq1[0], mseq2[0], localhomtable[i]+j-(njob-nadd), off1, off2, (int)pscore, strlen( mseq1[0] ), 'h' );
+						else if( !specifictarget && alg != 'S' && alg != 'V' )
+							putlocalhom2( mseq1[0], mseq2[0], localhomtable[i]+j-i, off1, off2, (int)pscore, strlen( mseq1[0] ), 'h' );
+						else
+						{
+							if( targetmap[i] != -1 && targetmap[j] != -1 )
+							{
+								putlocalhom2( mseq1[0], mseq2[0], localhomtable[targetmap[i]]+j, off1, off2, (int)pscore, strlen( mseq1[0] ), 'h' );
+								putlocalhom2( mseq2[0], mseq1[0], localhomtable[targetmap[j]]+i, off2, off1, (int)pscore, strlen( mseq2[0] ), 'h' ); // sukoshi muda.
+							}
+							else if( targetmap[j] != -1 )
+								putlocalhom2( mseq2[0], mseq1[0], localhomtable[targetmap[j]]+i, off2, off1, (int)pscore, strlen( mseq2[0] ), 'h' );
+							else if( targetmap[i] != -1 )
+								putlocalhom2( mseq1[0], mseq2[0], localhomtable[targetmap[i]]+j, off1, off2, (int)pscore, strlen( mseq1[0] ), 'h' );
+							else
+							{
+								reporterr( "okashii\n" );
+								exit( 1 );
+							}
+						}
 					}
 
 					pscore = score2dist( pscore, selfscore[i], selfscore[j] );
@@ -2820,7 +2944,7 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **d
 				if( store_dist) 
 				{
 					if( alg == 'Y' || alg == 'r' ) distancemtx[i][j-(njob-nadd)] = pscore;
-					else distancemtx[i][j] = pscore;
+					else distancemtx[i][j-i] = pscore;
 				}
 			}
 		}
@@ -2828,24 +2952,28 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **d
 	}
 
 
-	if( store_dist )
+	if( store_dist && ngui == 0 )
 	{
 		hat2p = fopen( hat2file, "w" );
 		if( !hat2p ) ErrorExit( "Cannot open hat2." );
 		if( alg == 'Y' || alg == 'r' )
 			WriteHat2_part_pointer( hat2p, njob, nadd, name, distancemtx );
 		else
-			WriteHat2_pointer( hat2p, njob, name, distancemtx );
+//			WriteHat2_pointer( hat2p, njob, name, distancemtx );
+			WriteFloatHat2_pointer_halfmtx( hat2p, njob, name, distancemtx ); // jissiha double
 		fclose( hat2p );
 	}
 
 	hat3p = fopen( "hat3", "w" );
 	if( !hat3p ) ErrorExit( "Cannot open hat3." );
-	if( store_localhom )
+	if( store_localhom && ngui == 0 )
 	{
+
 		fprintf( stderr, "\n\n##### writing hat3\n" );
 		if( alg == 'Y' || alg == 'r' )
 			ilim = njob-nadd;	
+		else if( specifictarget )
+			ilim = ntarget;
 		else
 			ilim = njob-1;	
 		for( i=0; i<ilim; i++ ) 
@@ -2855,10 +2983,15 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **d
 				jst = njob-nadd;
 				jj = 0;
 			}
+			else if( specifictarget )
+			{
+				jst = 0;
+				jj = 0;
+			}
 			else
 			{
-				jst = i+1;
-				jj = i+1;
+				jst = i;
+				jj = 0;
 			}
 			for( j=jst; j<njob; j++, jj++ )
 			{
@@ -2870,21 +3003,27 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **d
 //					if( alg == 'B' || alg == 'T' )
 //						fprintf( hat3p, "%d %d %d %7.5f %d %d %d %d %p\n", i, j, tmpptr->overlapaa, 1.0, tmpptr->start1, tmpptr->end1, tmpptr->start2, tmpptr->end2, (void *)tmpptr->next ); 
 //					else
-						fprintf( hat3p, "%d %d %d %7.5f %d %d %d %d h\n", i, j, tmpptr->overlapaa, tmpptr->opt, tmpptr->start1, tmpptr->end1, tmpptr->start2, tmpptr->end2 );
+					if( targetmap[j] == -1 || targetmap[i] < targetmap[j] )
+						fprintf( hat3p, "%d %d %d %7.5f %d %d %d %d h\n", targetmapr[i], j, tmpptr->overlapaa, tmpptr->opt, tmpptr->start1, tmpptr->end1, tmpptr->start2, tmpptr->end2 );
 //						fprintf( hat3p, "%d %d %d %7.5f %d %d %d %d h\n", i, j, tmpptr->overlapaa, tmpptr->opt, tmpptr->start1, tmpptr->end1, tmpptr->start2+1, tmpptr->end2+1 ); // zettai dame!!!!
 				}
 			}
 		}
+//		if( ngui == 0 )
+//		{
 #if DEBUG
-		fprintf( stderr, "calling FreeLocalHomTable\n" );
+			fprintf( stderr, "calling FreeLocalHomTable\n" );
 #endif
-		if( alg == 'Y' || alg == 'r' )
-			FreeLocalHomTable_part( localhomtable, (njob-nadd), nadd );
-		else
-			FreeLocalHomTable( localhomtable, njob );
+			if( alg == 'Y' || alg == 'r' )
+				FreeLocalHomTable_part( localhomtable, (njob-nadd), nadd );
+			else if( specifictarget )
+				FreeLocalHomTable_part( localhomtable, ntarget, njob );
+			else
+				FreeLocalHomTable_half( localhomtable, njob );
 #if DEBUG
-		fprintf( stderr, "done. FreeLocalHomTable\n" );
+			fprintf( stderr, "done. FreeLocalHomTable\n" );
 #endif
+//		}
 	}
 	fclose( hat3p );
 
@@ -2914,12 +3053,14 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **d
 	}
 	free( distseq1 );
 	free( distseq2 );
-	if( store_dist ) FreeDoubleMtx( distancemtx );
+	if( store_dist && ngui == 0 ) FreeDoubleHalfMtx( distancemtx, njob );
+
+	free( targetmap );
+	free( targetmapr );
 }
 
-	 
 
-int main( int argc, char *argv[] )
+int pairlocalalign( int ngui, int lgui, char **namegui, char **seqgui, double **distancemtx, LocalHom **localhomtable, int argc, char **argv )
 {
 	int  *nlen, *thereisxineachseq;
 	char **name, **seq;
@@ -2933,38 +3074,40 @@ int main( int argc, char *argv[] )
 	int alloclen;
 	Lastresx **lastresx;
 
-	arguments( argc, argv );
-#ifndef enablemultithread
-	nthread = 0;
-#endif
-	fprintf( stderr, "lastonce = %d\n", lastonce );
+//	reporterr( "argc=%d, argv[0]=%s\n", argc, argv[0] );
 
-	if( inputfile )
+	arguments( argc, argv );
+
+
+	if( !ngui )
 	{
-		infp = fopen( inputfile, "r" );
-		if( !infp )
+		if( inputfile )
 		{
-			fprintf( stderr, "Cannot open %s\n", inputfile );
+			infp = fopen( inputfile, "r" );
+			if( !infp )
+			{
+				fprintf( stderr, "Cannot open %s\n", inputfile );
+				exit( 1 );
+			}
+		}
+		else
+			infp = stdin;
+	
+		getnumlen( infp );
+		rewind( infp );
+	
+		if( njob < 2 )
+		{
+			fprintf( stderr, "At least 2 sequences should be input!\n"
+							 "Only %d sequence found.\n", njob ); 
+			exit( 1 );
+		}
+		if( njob > M )
+		{
+			fprintf( stderr, "The number of sequences must be < %d\n", M );
+			fprintf( stderr, "Please try the splittbfast program for such large data.\n" );
 			exit( 1 );
 		}
-	}
-	else
-		infp = stdin;
-
-	getnumlen( infp );
-	rewind( infp );
-
-	if( njob < 2 )
-	{
-		fprintf( stderr, "At least 2 sequences should be input!\n"
-						 "Only %d sequence found.\n", njob ); 
-		exit( 1 );
-	}
-	if( njob > M )
-	{
-		fprintf( stderr, "The number of sequences must be < %d\n", M );
-		fprintf( stderr, "Please try the splittbfast program for such large data.\n" );
-		exit( 1 );
 	}
 
 	if( ( alg == 'r' || alg == 'R' ) && dorp == 'p' )
@@ -2974,16 +3117,26 @@ int main( int argc, char *argv[] )
 	}
 
 	alloclen = nlenmax*2;
-	seq = AllocateCharMtx( njob, alloclen+10 );
+	if( ngui ) 
+	{
+		seq = seqgui;
+		name = namegui;
+	}
+	else
+	{
+		seq = AllocateCharMtx( njob, alloclen+10 );
+		name = AllocateCharMtx( njob, B );
+	}
+
 	aseq = AllocateCharMtx( 2, alloclen+10 );
 	bseq = AllocateCharMtx( njob, alloclen+10 );
 	dseq = AllocateCharMtx( njob, alloclen+10 );
 	mseq1 = AllocateCharMtx( njob, 0 );
 	mseq2 = AllocateCharMtx( njob, 0 );
-	name = AllocateCharMtx( njob, B );
 	nlen = AllocateIntVec( njob );
 	thereisxineachseq = AllocateIntVec( njob );
 
+
 	if( alg == 'R' )
 	{
 		lastresx = calloc( njob+1, sizeof( Lastresx * ) );
@@ -3025,9 +3178,12 @@ int main( int argc, char *argv[] )
 #if 0
 	Read( name, nlen, seq );
 #else
-	readData_pointer( infp, name, nlen, seq );
+	if( !ngui ) 
+	{
+		readData_pointer( infp, name, nlen, seq );
+		fclose( infp );
+	}
 #endif
-	fclose( infp );
 
 	constants( njob, seq );
 
@@ -3051,19 +3207,22 @@ int main( int argc, char *argv[] )
 //	writePre( njob, name, nlen, seq, 0 );
 
 
+
+
 	for( i=0; i<njob; i++ ) 
 	{
 		gappick0( bseq[i], seq[i] );
 		thereisxineachseq[i] = removex( dseq[i], bseq[i] );
 	}
 
-	pairalign( name, nlen, bseq, aseq, dseq, thereisxineachseq, mseq1, mseq2, alloclen, lastresx );
+	pairalign( name, nlen, bseq, aseq, dseq, thereisxineachseq, mseq1, mseq2, alloclen, lastresx, distancemtx, localhomtable, ngui );
 
 	fprintf( trap_g, "done.\n" );
 #if DEBUG
 	fprintf( stderr, "closing trap_g\n" );
 #endif
 	fclose( trap_g );
+	fclose( prep_g );
 
 //	writePre( njob, name, nlen, aseq, !contin );
 #if 0
@@ -3102,15 +3261,47 @@ int main( int argc, char *argv[] )
 		free( lastresx );
 	}
 #endif
-	FreeCharMtx( seq );
+	if( ngui == 0 ) 
+	{
+		FreeCharMtx( seq );
+		FreeCharMtx( name );
+	}
 	FreeCharMtx( aseq );
 	FreeCharMtx( bseq );
 	FreeCharMtx( dseq );
-	FreeCharMtx( name );
 	free( mseq1 );
 	free( mseq2 );
 	free( nlen );
 	free( thereisxineachseq );
+	freeconstants();
+
+	if( !ngui )
+	{
+		FreeCommonIP();
+	}
+	Falign( NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, 0, NULL );
+	G__align11( NULL, NULL, NULL, 0, 0, 0 ); // 20130603
+	G__align11_noalign( NULL, 0, 0, NULL, NULL, 0 );
+	L__align11( NULL, 0.0, NULL, NULL, 0, NULL, NULL );
+	L__align11_noalign( NULL, NULL, NULL );
+	genL__align11( NULL, NULL, NULL, 0, NULL, NULL );
+
+#if SHISHAGONYU
+	if( ngui )
+	{
+		char buf[100];
+		for( i=0; i<njob-1; i++ ) for( j=i+1; j<njob; j++ )
+		{
+			sprintf( buf, "%5.3f", distancemtx[i][j-i] );
+			distancemtx[i][j-i] = 0.0;
+			sscanf( buf, "%lf", distancemtx[i]+j-i );
+//			distancemtx[i][j-i] = 0.001 * (int)(distancemtx[i][j-i] * 1000 + 0.5);
+		}
+
+	}
+#endif
+
 
 	return( 0 );
 }
+
