@@ -585,6 +585,7 @@ void arguments( int argc, char *argv[] )
 	picksize = NOTSPECIFIED;
 	tokyoripara = NOTSPECIFIED;
 	legacygapcost = 0;
+	nwildcard = 0;
 
     while( --argc > 0 && (*++argv)[0] == '-' )
 	{
@@ -764,6 +765,9 @@ void arguments( int argc, char *argv[] )
 					fftWinSize = myatoi( *++argv );
 					--argc;
 					goto nextoption;
+				case ':':
+					nwildcard = 1;
+					break;
                 default:
                     fprintf( stderr, "illegal option %c\n", c );
                     argc = 0;
